import pygame

import data
import engine
import items
import rpg

class ChooseFromTenState(engine.State):
    def __init__(self, pgs, ten,title):
        engine.State.__init__(self, pgs.driver)
        self.pgs = pgs
        self.ten = ten
        self.chosen = None
        
        rect = pygame.Rect(10,10, 0, 0)
        mainbox = engine.Box(rect)
        
        x = 13
        y = 13
        
        intro = engine.Label(title)
        intro.rect.topleft = (x, y)
        w = intro.rect.w
        y += intro.rect.h + 5
        
        mainbox.add(intro)
        
        self.labels = []
        
        for index in range(10):
            try:
                skill = self.ten[index]
            except IndexError:
                skill = None
            texindex =  ((index + 1) % 10)
            text = "%d: " % texindex
            if skill:
                text += str(skill)
            label = engine.Button(text, self.assign, str(texindex))
            label.rect.topleft = [ x, y ]
            if label.rect.w > w: w = label.rect.w
            y += label.rect.h + 3
            mainbox.add(label)
            self.labels.append(label)
            
        y += 5
        cancelButton = engine.Button("(c)ancel", self.cancel, 'c')
        cancelButton.rect.topleft = [x,y]
        mainbox.add(cancelButton)
        y += cancelButton.rect.h + 3
            
        mainbox.rect.w = w
        mainbox.rect.h = y - rect.y 
                    
        self.add(mainbox)
        
    def assign(self, button):
        self.chosen = self.labels.index(button)
        self.cancel(button)
    
    def cancel(self, button):
        self.driver.pop()
        
    def paint(self, screen):
        self.pgs.paint(screen)
        engine.State.paint(self,screen)
  
class InventoryState(engine.State):
    def __init__(self, pgs):
        engine.State.__init__(self, pgs.driver)
        self._active = None
        self.aboutToAssign = None
        
        self.pgs = pgs
        w,h = pgs.driver.screen.getSize()
        
        rect = pygame.Rect( 20, 20, w-40,h-40 )
        mainbox = engine.Box(rect)
        
        equipLabel = engine.Label("Equipped")
        equipLabel.rect.topleft = [ rect.x + 3, rect.y + 3]
        mainbox.add(equipLabel)
        
        inventoryLabel = engine.Label("Inventory")
        inventoryLabel.rect.topleft = [ w/2 + 3, rect.y + 3]
        mainbox.add(inventoryLabel)
        
        # Eqipment
        y = equipLabel.rect.bottom + 10
        # For now, this is hard-coded.  May want to change later
        self.wpnLabel = engine.Label("Weapon: %s" % pgs.pc.weapon)
        self.wpnLabel.rect.topleft = [ rect.x + 3, y ]
        y = self.wpnLabel.rect.bottom + 3
        mainbox.add(self.wpnLabel)
        
        armorLabel = engine.Label("Armor:")
        armorLabel.rect.topleft = [ rect.x + 3, y ]
        y = armorLabel.rect.bottom + 3
        mainbox.add(armorLabel)
        
        self.armorLabels = {}
        for slot in pgs.pc.armor.keys():
            armor = pgs.pc.armor[slot]
            label = engine.Label("  %s : %s" % (slot, armor))
            label.rect.topleft = [ rect.x + 3, y ]
            y = label.rect.bottom + 3
            mainbox.add(label)
            self.armorLabels[slot] = label
            
        if not pgs.pc.armor:
            label = engine.Label("  None")
            label.rect.topleft = [ rect.x + 3, y]
            mainbox.add(label)
            self.armorLabels['None'] = label
            y = label.rect.bottom + 3
        
        # Display inventory
        y = equipLabel.rect.bottom + 10
        self.inventoryLabels = []
        for index, x in enumerate(data.hotkeys):
            name = ""
            if index < len(pgs.pc.inventory):
                name = pgs.pc.inventory[index]
            label = engine.Button("%d: %s" % (x, name),self.inventory,str(x))
            label.rect.topleft = [ w/2 + 3, y]
            self.inventoryLabels.append(label)
            mainbox.add(label)
            y = label.rect.bottom + 3
        
        # Options
        y += 10
        exitButton = engine.Button("ex(i)t", self.exit,'i')
        exitButton.rect.topleft = [ rect.x + 3, y ]
        mainbox.add(exitButton)
        
        x = exitButton.rect.right + 3
        equipButton = engine.Button("(e)quip", self.equip,'e')
        equipButton.rect.topleft = [ x, y ]
        mainbox.add(equipButton)
        self.equipButton = equipButton
        
        x = equipButton.rect.right + 3
        assignButton = engine.Button("(a)ssign to hotkey", self.assign,'a')
        assignButton.rect.topleft = [x, y]
        mainbox.add(assignButton)
        self.assignButton = assignButton
        
        x = assignButton.rect.right + 3
        dropButton = engine.Button("(d)rop", self.drop, 'd')
        dropButton.rect.topleft = [x,y]
        mainbox.add(dropButton)
        self.dropButton = dropButton
        
        y = exitButton.rect.bottom + 3
        
        mainbox.rect.height = y - mainbox.rect.y
        
        self.add(mainbox)
 
    def activate(self):
        if self.aboutToAssign:
            assigned = self.which.chosen
            if assigned != None:
                item = self.pgs.pc.inventory[self.aboutToAssign]
                skill = items.UseItemSkill("Use %s" % item.name,
                                          self.pgs.pc,
                                          item)
                self.pgs.hotkeys[assigned] = skill
                
        self.active = None
        self.aboutToAssign = None
 
    def assign(self, button):
        active = self.active
        if active and active in self.inventoryLabels:
            index = self.getPickedInventory(active)
            if index != None:
                self.aboutToAssign = index
                self.which = ChooseFromTenState(self.pgs, self.pgs.hotkeys,
                                                "Choose which hotkey to assign")
                self.driver.push(self.which)
            else:
                self.active = None
        else:
            self.active = button
 
    def drop(self, button):
        active = self.active
        if active and active in self.inventoryLabels:
            index = self.getPickedInventory(active)
            if index != None:
                item = self.pgs.pc.inventory[index]
                tilemap = self.pgs.tiles
                loot = rpg.Loot(tilemap, item)
                loot.world.center = self.pgs.pc.world.center
                tilemap.addLoot(loot)
                self.pgs.pc.deequip(item)
                self.pgs.pc.inventory.remove(item)
                self.refresh()
        else:
            self.active = button
 
    def equip(self, button):
        active = self.active
        if active and active in self.inventoryLabels:
            index = self.getPickedInventory(active)
            if index != None:
                self.pgs.pc.equip(self.pgs.pc.inventory[index])
                self.refresh()
            else:
                self.active = None
        else:
            self.active = button
        
    def exit(self, button):
        self.driver.pop()
 
    def getActive(self):
        return self._active
    
    def getPickedInventory(self, active):
        index = int(active.text[0]) - 1
        if index == 0: index = 10
        if index < len(self.pgs.pc.inventory):
            return index
        else:
            return None
        
    def inventory(self, button):
        oldActive = self._active
        self.active = button
        if oldActive == self.equipButton:
            self.equip(button)
        elif oldActive == self.assignButton:
            self.assign(button)
        elif oldActive == self.dropButton:
            self.drop(button)
        
    def paint(self, screen):
        self.pgs.paint(screen)
        engine.State.paint(self,screen)
        
    def refresh(self):
        self.driver.pop()
        self.driver.push(InventoryState(self.pgs))
        
    def setActive(self, button):
        if self._active:
            self._active.background = None
        self._active = button
        if button: 
            button.background = ( 64, 64, 64 )
        
    active = property(getActive, setActive)
    
class GameOverState(engine.State):
    def __init__(self,pgs):
        engine.State.__init__(self, pgs.driver)
        
        rip = engine.Label("R.I.P. %s" % pgs.pc.name, (255, 64, 64), 32)
        rip.rect.topleft = (10,10)
        self.add(rip)
        y = rip.rect.bottom + 10
        x = 10
        
        desc = "A level %d %s, mercilessly slain %d stories above the earth"
        data = (pgs.pc.level, pgs.pc.progression.name, pgs.level)
        
        descLabel = engine.Label(desc % data)
        descLabel.rect.topleft = [x,y]
        self.add(descLabel)
        
        gameover = engine.Label("Game Over", (255, 255, 255), 64)
        gameover.rect.center = pgs.driver.screen.center
        self.add(gameover)
        
        quit = engine.Button("(q)uit", self.quit, 'q')
        quit.rect.bottomleft = [ 3, pgs.driver.screen.h - 3]
        self.add(quit)
        
    def quit(self, button):
        raise SystemExit
    
class ConfirmDialog(engine.State):
    def __init__(self, driver, message):
        engine.State.__init__(self, driver)
        
        box = engine.Box(pygame.Rect(0, 0, 0, 0))
        self.add(box)
        msg = engine.Label(message)
        box.add(msg)
        w = msg.rect.w
        
        yes = engine.Button("(y)es", self.yes,'y')
        if yes.rect.w > w: w = yes.rect.w
        box.add(yes)
        no = engine.Button("(n)o", self.no,'n')
        box.add(no)
        
        box.rect.w = w + 6
        box.rect.h = no.rect.h + msg.rect.h + 9
        box.rect.center = driver.screen.center
        
        msg.rect.topleft = ( box.rect.x + 3, box.rect.y + 3 )
        yes.rect.topleft = ( box.rect.x + 3, msg.rect.bottom + 3)
        no.rect.topleft = ( box.rect.x + box.rect.w - no.rect.w - 3,
                            msg.rect.bottom+3)
        
    def yes(self, button):
        self.confirmed = True
        self.driver.pop()
        
    def no(self, button):
        self.confirmed = False
        self.driver.pop()  